
//http://vec3.ca/bicubic-filtering-in-fewer-taps/

void BicubicTexCoord(vec2 iTc,
                     inout vec2 f,
                     inout vec2 tc0, inout vec2 tc1, inout vec2 tc2, inout vec2 tc3)
{
    //get into the right coordinate system
    //vec2 texSize = vec2(textureSize(en_texModel, 0).xy);
    //vec2 invTexSize = vec2(1.0) / texSize;
    
    iTc *= texSize.xy;
    
    //round tc *down* to the nearest lower *texel center*
	//4.2 => 3.7 => 3 => 3.5 - nearest lower texel center for 4.2 is 3.5
	//4.7 => 4.2 => 4 => 4.5 - nearest lower texel center for 4.7 is 4.5
    vec2 tc = floor(iTc - 0.5) + 0.5;
	
    
	//???? mam s tim neco delat ???
	//iTc is already "shifted" by 0.5 in reprojection
	//remove this 0.5 shift and round to texel center

    
    //compute the fractional offset from that texel center
    //to the actual coordinate we want to filter at
    
    f = iTc - tc;
    
    //get our texture coordinates
    
    tc0 = tc - vec2(1.0);
    tc1 = tc;
    tc2 = tc + vec2(1.0);
    tc3 = tc + vec2(2.0);
  
    
    //convert them to normalized coordinates
   
    tc0 *= texSize.zw;
    tc1 *= texSize.zw;
    tc2 *= texSize.zw;
    tc3 *= texSize.zw;
    
}

void BicubicWeights(vec2 f, inout vec4 wx, inout vec4 wy)
{    
	// - zkusit .... ... melo by byt "rychlejsi"
	/*
	vec2 fmpF1 = (1.0 - f);
		
    vec4 f2 = vec4(f * f, fmpF1 * fmpF1);
    vec4 f3 = vec4(f2.xy * f, f2.zw * fmpF1);
    
	vec4 w = (4.0 + 3.0 * f3 - 6.0 * f2);
	
	wx = vec4(f3.z, w.x, w.z, f3.x);
    wy = vec4(f3.w, w.y, w.w, f3.y);
	*/
	//
	
    vec2 f2 = f * f;
    vec2 f3 = f2 * f;
    
    //const float tmpDiv = (1.0 / 6.0);
    vec2 fmpF1 = (1.0 - f);
    vec2 f12 = fmpF1 * fmpF1;
    vec2 f13 = f12 * fmpF1;
    
    //compute the filter weights
    //vec2 w0 = (f13);
    vec2 w1 = (4.0 + 3.0 * f3 - 6.0 * f2);
    vec2 w2 = (4.0 + 3.0 * f13 - 6.0 * f12);
    //vec2 w3 = (f3);
    
    wx = vec4(f13.x, w1.x, w2.x, f3.x);
    wy = vec4(f13.y, w1.y, w2.y, f3.y);
    
}



void BilinearTexCoord(vec2 iTc,
                      inout vec2 f,
                      inout vec2 tc0, inout vec2 tc1, inout vec2 tc2, inout vec2 tc3)
{
       
    
    //https://www.opengl.org/discussion_boards/showthread.php/168506-manual-bilinear-filter
    
    //get into the right coordinate system
    //vec2 textureSize = vec2(textureSize(en_texModel, 0).xy);
    //vec2 invTexelSize = vec2(1.0) / texSize;
    
    iTc *= texSize.xy;
    
    
    //round tc *down* to the nearest lower *texel center*
	//4.2 => 3.7 => 3 => 3.5 - nearest lower texel center for 4.2 is 3.5
	//4.7 => 4.2 => 4 => 4.5 - nearest lower texel center for 4.7 is 4.5
    vec2 tc = floor(iTc - 0.5) + 0.5;
           
    f = iTc - tc;
    
    
    tc0 = tc;
    tc1 = tc + vec2(1.0, 0.0);
    tc2 = tc + vec2(0.0, 1.0);
    tc3 = tc + vec2(1.0, 1.0);
    
    
    tc0 *= texSize.zw;
    tc1 *= texSize.zw;
    tc2 *= texSize.zw;
    tc3 *= texSize.zw;
    
}

float UnpackValue_BicubicInterpolation_NonLinearTexValues(vec2 iTc)
{
    
    vec2 f = vec2(0.0);
    
    
    vec2 tc0 = vec2(0.0);
    vec2 tc1 = vec2(0.0);
    vec2 tc2 = vec2(0.0);
    vec2 tc3 = vec2(0.0);
    
    vec4 wx = vec4(0.0);
    vec4 wy = vec4(0.0);
    
    BicubicTexCoord(iTc, f, tc0, tc1, tc2, tc3);
    BicubicWeights(f, wx, wy);
    
    float f00 = TEX2D(vec2(tc0.x, tc0.y)).r;
    float f10 = TEX2D(vec2(tc1.x, tc0.y)).r;
    float f20 = TEX2D(vec2(tc2.x, tc0.y)).r;
    float f30 = TEX2D(vec2(tc3.x, tc0.y)).r;
    
    float f01 = TEX2D(vec2(tc0.x, tc1.y)).r;
    float f11 = TEX2D(vec2(tc1.x, tc1.y)).r;
    float f21 = TEX2D(vec2(tc2.x, tc1.y)).r;
    float f31 = TEX2D(vec2(tc3.x, tc1.y)).r;
    
    float f02 = TEX2D(vec2(tc0.x, tc2.y)).r;
    float f12 = TEX2D(vec2(tc1.x, tc2.y)).r;
    float f22 = TEX2D(vec2(tc2.x, tc2.y)).r;
    float f32 = TEX2D(vec2(tc3.x, tc2.y)).r;
    
    float f03 = TEX2D(vec2(tc0.x, tc3.y)).r;
    float f13 = TEX2D(vec2(tc1.x, tc3.y)).r;
    float f23 = TEX2D(vec2(tc2.x, tc3.y)).r;
    float f33 = TEX2D(vec2(tc3.x, tc3.y)).r;
    
    vec4 cf0 = vec4(ConvertPackedValue(f00 * 255.0), ConvertPackedValue(f10 * 255.0),
                    ConvertPackedValue(f20 * 255.0), ConvertPackedValue(f30 * 255.0));
    vec4 cf1 = vec4(ConvertPackedValue(f01 * 255.0), ConvertPackedValue(f11 * 255.0),
                    ConvertPackedValue(f21 * 255.0), ConvertPackedValue(f31 * 255.0));
    vec4 cf2 = vec4(ConvertPackedValue(f02 * 255.0), ConvertPackedValue(f12 * 255.0),
                    ConvertPackedValue(f22 * 255.0), ConvertPackedValue(f32 * 255.0));
    vec4 cf3 = vec4(ConvertPackedValue(f03 * 255.0), ConvertPackedValue(f13 * 255.0),
                    ConvertPackedValue(f23 * 255.0), ConvertPackedValue(f33 * 255.0));
    
    return (1.0 / 36.0) * (
                           wy.x * dot(cf0, wx) +
                           wy.y * dot(cf1, wx) +
                           wy.z * dot(cf2, wx) +
                           wy.w * dot(cf3, wx)
                           );
}

float UnpackValue_BilinearInterpolation_NonLinearTexValues(vec2 iTc)
{
    vec2 f = vec2(0.0);
    
    
    vec2 tc0 = vec2(0.0);
    vec2 tc1 = vec2(0.0);
    vec2 tc2 = vec2(0.0);
    vec2 tc3 = vec2(0.0);
    
    
    BilinearTexCoord(iTc, f, tc0, tc1, tc2, tc3);
    
    
    float f0 = TEX2D( tc0 ).r;
    float f1 = TEX2D( tc1 ).r;
    float f2 = TEX2D( tc2 ).r;
    float f3 = TEX2D( tc3 ).r;
    
    float unpackedValue0 = ConvertPackedValue(f0 * 255.0);
    float unpackedValue1 = ConvertPackedValue(f1 * 255.0);
    float unpackedValue2 = ConvertPackedValue(f2 * 255.0);
    float unpackedValue3 = ConvertPackedValue(f3 * 255.0);
    
    //compute the fractional offset from that texel center
    //to the actual coordinate we want to filter at
    
    
    //https://en.wikipedia.org/wiki/Bilinear_filtering
    //float v = (unpackedValue0 * (1.0 - f.x) + unpackedValue1 * f.x) * (1.0 - f.y) +
    //            (unpackedValue2 * (1.0 - f.x) + unpackedValue3 * f.x) * f.y;
    
    float tA = mix( unpackedValue0, unpackedValue1, f.x );
    float tB = mix( unpackedValue2, unpackedValue3, f.x );
    float v = mix( tA, tB, f.y );
    
    
    return v; 
}
