
//====================================================
// Constants

#define PI_2 1.57079632679
#define EXP 2.71828182846

//====================================================
// Precalculated projection variables

//-wPadding + wAr * minPixelOffsetX
//1.0 / wAR
//-h + hPadding - hAr * minPixelOffsetY
//1.0 / -hAR
uniform vec4 mercatorPrecomp;

//wPadding - minPixelOffsetX * wAr
//wAr
//h - hPdadding + minPixelOffsetY * hAr
//hAr
uniform vec4 equirectPrecomp;

//w, h, 1/w, 1/h
uniform vec4 texSize;

//tileW, tileH
uniform vec2 tileFrame;

//=====================================================

vec2 MercatorProjectInverse(vec2 p)
{
	vec2 tmp = vec2((p.x + mercatorPrecomp.x) * mercatorPrecomp.y,	
				    (p.y + mercatorPrecomp.z) * mercatorPrecomp.w);
	
	//MercatorProjectInverseInternal
	tmp.y = 2.0 * atan(pow(EXP, tmp.y)) - PI_2;
	
	return tmp;
}

vec2 EquirectangularProject(vec2 rawPixel)
{	
	return vec2(equirectPrecomp.x + rawPixel.x * equirectPrecomp.y,
				equirectPrecomp.z - rawPixel.y * equirectPrecomp.w);
}

vec2 GetCoordinateFromWorldPos(vec2 worldPos)
{
	// clamping to compensate "world" repeating		
	vec2 pPos;
	//pPos.x = mod(worldPos.x, 1.0);
	pPos.x = worldPos.x;
	pPos.y = 1.0 - worldPos.y;

	return MercatorProjectInverse(pPos);
}

vec2 FromEquirectangularToMercator(vec2 worldPos)
{
	vec2 gps = GetCoordinateFromWorldPos(worldPos);
	vec2 pixel = EquirectangularProject(gps);

    //vypne interpolaci !!!
    //round "pixel" to whole numbers
    //use floor - we have only + numbers, so its OK
    // 3.7 => 4; 3.2 => 3
    //WIN funguje, iOS ne :(
    //vec2 texel = floor(pixel + vec2(0.5)) + vec2(0.5);

	vec2 texel = pixel + vec2(0.50, 0.50);
    
	//border check at the end - clamp to last pixel if it is out of tile
	texel = clamp(texel, vec2(0.0), tileFrame);
	
    texel *= texSize.zw;
      
	return texel;
}


