precision highp float;



#define TEX2D(pos) texture2D(en_texModel, pos)

//======================================================================
//Skeleton for ventusky methods with 1 parametr
//only one input texture is used
// (basically all except wind)
//======================================================================


varying vec2 reprojTx; //pro projekce
varying vec2 vWorldPos; //pro projekce


uniform vec4 minMaxStepOpacity;

uniform sampler2D en_texModel;
uniform sampler2D colorMap;


uniform sampler2D hatchMap;
//uniform sampler2D stormMap;
uniform float visibleZoom;
uniform float screenScale;

//uniform vec2 texSize;
uniform float enableInterpolation;


float ConvertPackedValue(float b) 
{
    /*
    b = b - 0.0392156;
    //b = clamp(b - 0.0392156, 0.0, 1.0)
    
    float s = 0.0;
    if (b > 0.94118) s = b - 9.1066667;
    else if (b > 0.352941) s = b - 0.290196;
    else if (b < 0.0) s = 0.0;
    else if (b < 0.0392156) s = b / 0.0392156;
    else s = b - 0.0392156;
    */
    
    
    //b *= 255.0;
    b = b - 10.0;
	//b = clamp(b - 10.0, 0.0, 255.0);
	
	//https://stackoverflow.com/questions/47597588/glsl-optimize-if-else

	float s = 0.0;
	if (b > 240.0) s = 100.0 * b - 2520.0;
	else if (b > 90.0) s = 5.0 * b - 370.0;
    else if (b >= 10.0) s = b - 10.0;
    else if (b > 0.0) s = b / 10.0;
   
	
    /*
     
     var s = 0.0;
     if (b > 240.0) s = 780.0 + (b - 10.0 - 230.0) * 100.0;
     else if (b > 90.0) s = 80.0 + (b - 10.0 - 80.0) * 5.0;
     else if (b < 10.0) s = b / 10.0
     else s = b - 10.0;
     */
    
	return s;
}

//#my_include "ventusky/ventusky_layer_reprojection.ps.glsl"
#my_include "ventusky/ventusky_layer_interpolations.ps.glsl"

vec4 GetUnpackedColor(float val)
{			
    float tx = (val - minMaxStepOpacity.x) / (minMaxStepOpacity.y - minMaxStepOpacity.x);
		
    //float tx = (val) / (minMaxStepOpacity.y / 255.0);
    
	return texture2D(colorMap, vec2(tx, 0.5));
}





vec4 PrepareBilinearForTypes(vec2 iTc, out vec2 f)
{	
	vec2 tc0 = vec2(0.0);
	vec2 tc1 = vec2(0.0);
	vec2 tc2 = vec2(0.0);
	vec2 tc3 = vec2(0.0);


	BilinearTexCoord(iTc, f, tc0, tc1, tc2, tc3);

	vec4 val = vec4(0.0);
	
#ifdef GLES_2	
	val.x = TEX2D(tc0).a;
	val.y = TEX2D(tc1).a;
	val.z = TEX2D(tc2).a;
	val.w = TEX2D(tc3).a;
#else	
	val.x = TEX2D(tc0).g;
	val.y = TEX2D(tc1).g;
	val.z = TEX2D(tc2).g;
	val.w = TEX2D(tc3).g;
#endif



	return val * 255.0;
}



float BilinearInterpolationForType(vec4 unpackedValue, vec2 f, float type)
{	
	vec4 tmp = abs(unpackedValue - vec4(type));
	unpackedValue = vec4(lessThan(tmp, vec4(1.0))) * vec4(type);
	
	float tA = mix(unpackedValue.x, unpackedValue.y, f.x);
	float tB = mix(unpackedValue.z, unpackedValue.w, f.x);
	return mix(tA, tB, f.y);	
}

/*
float BilinearInterpolation_Storm(vec4 unpackedValue, vec2 f)
{
	return BilinearInterpolationForType(unpackedValue, f, 40.0);
}
 */

float BilinearInterpolation_Snow(vec4 unpackedValue, vec2 f)
{
	//if value is 20 (= convective snow) - replace it with 30 (= snow)
	vec4 tmp = abs(unpackedValue - vec4(20.0));
	
	if (tmp.x < 1.0) unpackedValue.x = 30.0;
	if (tmp.y < 1.0) unpackedValue.y = 30.0;
	if (tmp.z < 1.0) unpackedValue.z = 30.0;
	if (tmp.w < 1.0) unpackedValue.w = 30.0;

	return BilinearInterpolationForType(unpackedValue, f, 30.0);	
}

void main()
{		
	
	float unpackedValue = 0.0;
		
	//vec2 reprojTx = FromEquirectangularToMercator(vWorldPos);

   
	if (enableInterpolation < 0.5)
	{
        float packedValue = TEX2D(reprojTx).r;
		unpackedValue = ConvertPackedValue(packedValue * 255.0);
	}
	else 
	{        
		#ifdef USE_LOW_QUALITY
			unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(reprojTx);
		#else			
			unpackedValue = UnpackValue_BicubicInterpolation_NonLinearTexValues(reprojTx);
		#endif    
	}
    

	
	vec2 f = vec2(0.0);
	vec4 unpackedType = PrepareBilinearForTypes(reprojTx, f);
	float typeSnow = BilinearInterpolation_Snow(unpackedType, f);	
	//float typeStorm = BilinearInterpolation_Storm(unpackedType, f);

	//no interpolation for storm
	//float tmpStorm = abs(unpackedType.x - 40.0);
	//float typeStorm = (tmpStorm < 1.0) ? 40.0 : 0.0;
	
		
    
    //vec4 stormColor = vec4(0.0);
	/*
	if (typeStorm > 20.0)
	{
		stormColor.a = texture2D(stormMap, 30.0 * visibleMult * vWorldPos.xy).r;
		stormColor = vec4(0.82, 0.04, 0.04, 1.0) * stormColor.a;
		//stormColor = vec4(stormColor.a);
	}
	*/
    
    vec4 snowColor = vec4(0.0);
	if (typeSnow > 15.0)
	{
        float visibleMult = pow(2.0, visibleZoom);
        
		snowColor = texture2D(hatchMap, 40.0 * screenScale * visibleMult * vWorldPos.xy).rrrr;
	}


	vec4 color = GetUnpackedColor(unpackedValue);
	
	color = snowColor + color * (1.0 - snowColor.a);
	//color = stormColor + color * (1.0 - stormColor.a);

	gl_FragColor = color;
	gl_FragColor.a *= minMaxStepOpacity.w;	//*= multiply opacity from "color map" with "user defined"		
}
